@@ -23,16 +23,21 @@
 
 #include <glom/libglom/data_structure/field.h>
 
+namespace Glom
+{
+
 class CalcInProgress
 {
 public:
   CalcInProgress();
 
-  sharedptr<const Field> m_field;
+  sharedptr<const Field> m_field; 
   Gnome::Gda::Value m_value; //If it's been calculated.
   bool m_calc_in_progress;
   bool m_calc_finished;
 };
 
+} //namespace Glom
+
 #endif //GLOM_MODE_DATA_CALCINPROGRESS_H
 
