@@ -81,7 +81,10 @@ namespace Conversions
   Gnome::Gda::Value get_example_value(Field::glom_field_type field_type);
 
   ///Get a string representing binary data, for SQL
-  Glib::ustring get_escaped_binary_data(guint8* buffer, size_t buffer_size);
+  Glib::ustring escape_binary_data_postgres(guint8* buffer, size_t buffer_size);
+  Glib::ustring escape_binary_data_sqlite(guint8* buffer, size_t buffer_size);
+  guint8* unescape_binary_data_postgres(const Glib::ustring& escaped_binary_data, size_t& length);
+  guint8* unescape_binary_data_sqlite(const Glib::ustring& escaped_binary_data, size_t& length);
 
   Gnome::Gda::Value convert_value(const Gnome::Gda::Value& value, Field::glom_field_type target_glom_type);
 
