@@ -50,12 +50,12 @@ Dialog_Layout::~Dialog_Layout()
 {
 }
 
-void Dialog_Layout::set_document(const Glib::ustring& layout, Document_Glom* /* document */, const Glib::ustring& table_name, const type_vecLayoutFields& /* table_fields */)
+void Dialog_Layout::set_document(const Glib::ustring& layout_name, const Glib::ustring& layout_platform, Document_Glom* /* document */, const Glib::ustring& table_name, const type_vecLayoutFields& /* table_fields */)
 {
   m_modified = false;
 
-  m_layout_name = layout; 
-  //m_document = document;
+  m_layout_name = layout_name;
+  m_layout_platform = layout_platform;
   m_table_name = table_name;
 
   m_modified = false;
