@@ -32,7 +32,7 @@ public:
   Box_Data_Details_Find();
   virtual ~Box_Data_Details_Find();
 
-  virtual bool init_db_details(const Glib::ustring& table_name);
+  virtual bool init_db_details(const Glib::ustring& table_name, const Glib::ustring& layout_platform);
 
   virtual Gtk::Widget* get_default_button(); //override
 
