@@ -631,14 +631,20 @@ void Dialog_Layout_Report::on_button_add()
   Glib::RefPtr<type_model> model_available = Glib::RefPtr<type_model>::cast_dynamic(m_treeview_available_parts->get_model());
 
   Gtk::TreeModel::iterator parent = get_selected_group_parent();
-  sharedptr<LayoutItem> pParentPart;
+  sharedptr<const LayoutItem> pParentPart;
   if(parent)
-    pParentPart = (*parent)[m_model_available_parts_main->m_columns.m_col_item];
+  {
+    sharedptr<LayoutItem> temp = (*parent)[m_model_available_parts_main->m_columns.m_col_item];
+    pParentPart = temp;
+  }
 
   Gtk::TreeModel::iterator available = get_selected_available();
-  sharedptr<LayoutItem> pAvailablePart;
+  sharedptr<const LayoutItem> pAvailablePart;
   if(available)
-    pAvailablePart = (*available)[model_available->m_columns.m_col_item];
+  {
+    sharedptr<LayoutItem> temp = (*available)[model_available->m_columns.m_col_item];
+    pAvailablePart = temp;
+  }
 
 
   //Check whether the available part may be a child of the selected parent:
@@ -649,7 +655,9 @@ void Dialog_Layout_Report::on_button_add()
     parent = (*parent).parent();
     if(parent)
     {
-      pParentPart = (*parent)[model_available->m_columns.m_col_item];
+      sharedptr<LayoutItem> temp = (*parent)[model_available->m_columns.m_col_item];
+      pParentPart = temp;
+
       if(!TreeStore_ReportLayout::may_be_child_of(pParentPart, pAvailablePart))
         return; //Not allowed either.
     }
@@ -661,7 +669,7 @@ void Dialog_Layout_Report::on_button_add()
     Gtk::TreeModel::iterator iter;
     if(parent)
     {
-      m_treeview_parts_main->expand_row( Gtk::TreePath(parent), true);
+      m_treeview_parts_main->expand_row( Gtk::TreeModel::Path(parent), true);
       iter = model->append(parent->children());
     }
     else
@@ -671,7 +679,7 @@ void Dialog_Layout_Report::on_button_add()
   }
 
   if(parent)
-    treeview->expand_row( Gtk::TreePath(parent), true);
+    treeview->expand_row( Gtk::TreeModel::Path(parent), true);
 
   enable_buttons();
 }
@@ -789,7 +797,7 @@ void Dialog_Layout_Report::on_button_formatting()
         if(field_chosen)
         {
           *field = *field_chosen;
-          model->row_changed(Gtk::TreePath(iter), iter);
+          model->row_changed(Gtk::TreeModel::Path(iter), iter);
         }
       }
     }
@@ -842,7 +850,7 @@ void Dialog_Layout_Report::on_button_edit()
               {
                 *fieldsummary = *chosenitem; //TODO_Performance.
 
-                model->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+                model->row_changed(Gtk::TreeModel::Path(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
                 m_modified = true;
               }
             }
@@ -865,7 +873,7 @@ void Dialog_Layout_Report::on_button_edit()
           if(chosenitem)
           {
             *field = *chosenitem; //TODO_Performance.
-            model->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+            model->row_changed(Gtk::TreeModel::Path(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
             m_modified = true;
           }
         }
@@ -878,7 +886,7 @@ void Dialog_Layout_Report::on_button_edit()
             if(chosen)
             {
               *layout_item_text = *chosen;
-              model->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+              model->row_changed(Gtk::TreeModel::Path(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
               m_modified = true;
             }
           }
@@ -891,7 +899,7 @@ void Dialog_Layout_Report::on_button_edit()
               if(chosen)
               {
                 *layout_item_image = *chosen;
-                model->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+                model->row_changed(Gtk::TreeModel::Path(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
                 m_modified = true;
               }
             }
@@ -923,7 +931,7 @@ void Dialog_Layout_Report::on_button_edit()
                       if(chosenitem)
                       {
                         *group_by = *chosenitem;
-                        model->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+                        model->row_changed(Gtk::TreeModel::Path(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
                         m_modified = true;
                       }
                     }
