@@ -18,10 +18,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h" // For GLOM_ENABLE_MAEMO
+
 #include "adddel.h"
 #include <algorithm> //For std::find.
 #include <glibmm/i18n.h>
-#include "../cellrendererlist.h"
+#include "../cellrendererlist/cellrendererlist.h"
 #include "treeviewcolumn_glom.h"
 #include <glom/libglom/data_structure/glomconversions.h>
 #include "../../dialog_invalid_data.h"
@@ -29,6 +31,10 @@
 #include "bakery/App/App_Gtk.h"
 #include <iostream> //For debug output.
 
+#ifdef GLOM_ENABLE_MAEMO
+#include <hildonmm/note.h>
+#endif // GLOM_ENABLE_MAEMO
+
 #include "eggcolumnchooser/eggcolumnchooserdialog.h"
 
 namespace Glom
@@ -71,8 +77,7 @@ AddDelColumnInfo& AddDelColumnInfo::operator=(const AddDelColumnInfo& src)
 
 AddDel::AddDel()
 : m_col_key(0),
-  m_pColumnHeaderPopup(0),
-  m_allow_column_chooser(false),
+  m_pMenuPopup(0),
   m_auto_add(true),
   m_allow_add(true),
   m_allow_delete(true)
@@ -84,8 +89,7 @@ AddDel::AddDel()
 AddDel::AddDel(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& /* refGlade */)
 : Gtk::VBox(cobject),
   m_col_key(0),
-  m_pColumnHeaderPopup(0),
-  m_allow_column_chooser(false),
+  m_pMenuPopup(0),
   m_auto_add(true),
   m_allow_add(true),
   m_allow_delete(true)
@@ -119,7 +123,7 @@ void AddDel::init()
   //m_TreeView.set_column_drag_function( sigc::mem_fun(*this, &AddDel::on_treeview_column_drop) );
 
 
-  m_TreeView.add_events(Gdk::BUTTON_PRESS_MASK); //Allow us to catch button_press_event and button_release_event
+  //m_TreeView.add_events(Gdk::BUTTON_PRESS_MASK); //Allow us to catch button_press_event and button_release_event
   m_TreeView.signal_button_press_event().connect_notify( sigc::mem_fun(*this, &AddDel::on_treeview_button_press_event) );
 
   m_TreeView.signal_columns_changed().connect( sigc::mem_fun(*this, &AddDel::on_treeview_columns_changed) );
@@ -143,13 +147,20 @@ AddDel::~AddDel()
 
 void AddDel::warn_about_duplicate()
 {
-  Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Duplicate")), true, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK);
-  //TODO: dialog.set_transient_for(get_parent_window());
+  Glib::ustring message;
 
   if(m_prevent_duplicates_warning.empty())
-    dialog.set_secondary_text(_("This item already exists. Please try again."));
+    message = _("This item already exists. Please try again.");
   else
-    dialog.set_secondary_text(m_prevent_duplicates_warning); //Something more specific and helpful.
+    message = m_prevent_duplicates_warning; //Something more specific and helpful.
+
+#ifdef GLOM_ENABLE_MAEMO
+  Hildon::Note dialog(Hildon::NOTE_TYPE_INFORMATION, message);
+#else
+  Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Duplicate")), true, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK);
+  dialog.set_secondary_text(message);
+#endif
+  //TODO: dialog.set_transient_for(get_parent_window());
 
   dialog.run();
 }
@@ -225,28 +236,58 @@ void AddDel::on_MenuPopup_activate_Delete()
 
 void AddDel::setup_menu()
 {
-  //Clear existing menu items:
-  m_MenuPopup.items().clear();
+  m_refActionGroup = Gtk::ActionGroup::create();
+  m_refActionGroup->add(Gtk::Action::create("ContextMenu", "Context Menu") );
 
-  //Add new menu items:
-  Gtk::Menu_Helpers::MenuElem menuItem_Edit(_("Edit"));
-  m_MenuPopup.items().push_back(menuItem_Edit);
-  m_MenuPopup.items().back().signal_activate().connect(sigc::mem_fun(*this, &AddDel::on_MenuPopup_activate_Edit));
+  m_refContextEdit = Gtk::Action::create("ContextEdit", Gtk::Stock::EDIT);
+  m_refActionGroup->add(m_refContextEdit,
+    sigc::mem_fun(*this, &AddDel::on_MenuPopup_activate_Edit) );
 
   if(get_allow_user_actions())
   {
-    Gtk::Menu_Helpers::MenuElem menuItem_Delete(_("Delete"));
-    m_MenuPopup.items().push_back(menuItem_Delete);
-    m_MenuPopup.items().back().signal_activate().connect(sigc::mem_fun(*this, &AddDel::on_MenuPopup_activate_Delete));
-  }
-  
-  if(m_allow_column_chooser)
-  {
-    Gtk::Menu_Helpers::MenuElem menuitem_ChooseColumns(_("Choose columns"));
-    m_MenuPopup.items().push_back(menuitem_ChooseColumns);
-    m_MenuPopup.items().back().signal_activate().connect(sigc::mem_fun(*this, &AddDel::on_MenuPopup_activate_ChooseColumns));
+    m_refContextDelete =  Gtk::Action::create("ContextDelete", Gtk::Stock::DELETE);
+    m_refActionGroup->add(m_refContextDelete,
+      sigc::mem_fun(*this, &AddDel::on_MenuPopup_activate_Delete) );
   }
 
+  m_refUIManager = Gtk::UIManager::create();
+
+  m_refUIManager->insert_action_group(m_refActionGroup);
+
+  //TODO: add_accel_group(m_refUIManager->get_accel_group());
+
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  try
+  {
+#endif
+    Glib::ustring ui_info = 
+        "<ui>"
+        "  <popup name='ContextMenu'>"
+        "    <menuitem action='ContextEdit'/>"
+        "    <menuitem action='ContextDelete'/>"
+        "  </popup>"
+        "</ui>";
+
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+    m_refUIManager->add_ui_from_string(ui_info);
+  }
+  catch(const Glib::Error& ex)
+  {
+    std::cerr << "building menus failed: " <<  ex.what();
+  }
+#else
+  std::auto_ptr<Glib::Error> error;
+  m_refUIManager->add_ui_from_string(ui_info, error);
+  if(error.get() != NULL)
+  {
+    std::cerr << "building menus failed: " << error->what();
+  }
+#endif //GLIBMM_EXCEPTIONS_ENABLED
+
+  //Get the menu:
+  m_pMenuPopup = dynamic_cast<Gtk::Menu*>( m_refUIManager->get_widget("/ContextMenu") ); 
+  if(!m_pMenuPopup)
+    g_warning("menu not found");
 }
 
 bool AddDel::on_button_press_event_Popup(GdkEventButton *event)
@@ -256,14 +297,14 @@ bool AddDel::on_button_press_event_Popup(GdkEventButton *event)
   if(mods & GDK_BUTTON3_MASK)
   {
     //Give user choices of actions on this item:
-    m_MenuPopup.popup(event->button, event->time);
+    m_pMenuPopup->popup(event->button, event->time);
   }
   else
   {
     if(event->type == GDK_2BUTTON_PRESS)
     {
       //Double-click means edit.
-      //Disable this, because it is confusing when single-click activates editable cells too. on_MenuPopup_activate_Edit();
+      //Disable this, because it is confusing when single-click activates editable cells too.
     }
   }
 
@@ -660,7 +701,7 @@ void AddDel::construct_specified_columns()
           if(pCellRenderer)
           {
             //Make it editable:
-            pCellRenderer->property_editable() = true;
+            g_object_set(pCellRenderer->gobj(), "editable", TRUE, (gpointer)NULL);
 
             //Connect to its signal:
             pCellRenderer->signal_edited().connect(
@@ -672,7 +713,7 @@ void AddDel::construct_specified_columns()
            Gtk::CellRendererToggle* pCellRenderer = dynamic_cast<Gtk::CellRendererToggle*>(m_TreeView.get_column_cell_renderer(view_column_index));
            if(pCellRenderer)
            {
-             pCellRenderer->property_activatable() = true;
+             g_object_set(pCellRenderer->gobj(), "activatable", TRUE, (gpointer)NULL);
 
              //Connect to its signal:
              pCellRenderer->signal_toggled().connect(
@@ -681,14 +722,20 @@ void AddDel::construct_specified_columns()
         }
       }
 
-    ++view_column_index;
+      //Connect other signals:
+      Gtk::CellRenderer* pCellRenderer = m_TreeView.get_column_cell_renderer(view_column_index);
+      if(pCellRenderer)
+        pCellRenderer->signal_editing_started().connect(
+          sigc::bind( sigc::mem_fun(*this, &AddDel::on_treeview_cell_editing_started), model_column_index) );
 
+       ++view_column_index;
     } //is visible
 
     ++model_column_index;
   }
 
   //Delete the vector's items:
+  model_column_index = 0;
   for(type_vecModelColumns::iterator iter = vecModelColumns.begin(); iter != vecModelColumns.end(); ++iter)
   {
      Gtk::TreeModelColumnBase* pModelColumn = *iter;
@@ -706,6 +753,12 @@ void AddDel::construct_specified_columns()
           delete pModelColumnDerived;
           break;
         }
+        case(AddDelColumnInfo::STYLE_Numerical):
+        {
+          Gtk::TreeModelColumn<int>* pModelColumnDerived = static_cast< Gtk::TreeModelColumn<int>* >(pModelColumn);
+          delete pModelColumnDerived;
+          break;
+        }
         default:
         {
           Gtk::TreeModelColumn<Glib::ustring>* pModelColumnDerived = static_cast< Gtk::TreeModelColumn<Glib::ustring>* >(pModelColumn);
@@ -715,7 +768,13 @@ void AddDel::construct_specified_columns()
 
         *iter = 0;
       }
-     }
+    }
+    else
+    {
+      std::cerr << "debug: AddDel::construct_specified_columns(): Leaking a Gtk::TreeModelColumn<>." << std::endl;
+    }
+    
+    ++model_column_index;
   }
 
   m_TreeView.columns_autosize();
@@ -1011,9 +1070,10 @@ void AddDel::remove_item_by_key(const Glib::ustring& strKey)
 }
 
 AddDel::InnerIgnore::InnerIgnore(AddDel* pOuter)
+: m_pOuter(pOuter),
+  m_bPreventUserSignals(false),
+  m_bIgnoreSheetSignals(false)
 {
-  m_pOuter = pOuter;
-
   if(m_pOuter)
   {
     m_bPreventUserSignals = m_pOuter->get_prevent_user_signals();
@@ -1057,7 +1117,7 @@ void AddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int
   if(path_string.empty())
     return;
 
-  Gtk::TreePath path(path_string);
+  Gtk::TreeModel::Path path(path_string);
 
   //Get the row from the path:
   Gtk::TreeModel::iterator iter = m_refListStore->get_iter(path);
@@ -1132,7 +1192,7 @@ void AddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const Gli
   if(path_string.empty())
     return;
 
-  Gtk::TreePath path(path_string);
+  Gtk::TreeModel::Path path(path_string);
 
   //Get the row from the path:
   Gtk::TreeModel::iterator iter = m_refListStore->get_iter(path);
@@ -1244,6 +1304,19 @@ void AddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const Gli
   }
 }
 
+
+void AddDel::on_treeview_cell_editing_started(Gtk::CellEditable* /* editable */, const Glib::ustring& path_string, int model_column_index)
+{
+  Gtk::TreeModel::Path path(path_string);
+
+  if(!m_refListStore)
+    return;
+
+  Gtk::TreeModel::iterator iterRow = m_refListStore->get_iter(path);
+  if(iterRow)
+    signal_user_activated().emit(iterRow, model_column_index);
+}
+
 AddDel::type_signal_user_added AddDel::signal_user_added()
 {
   return m_signal_user_added;
@@ -1281,67 +1354,9 @@ AddDel::type_signal_user_reordered_columns AddDel::signal_user_reordered_columns
 
 void AddDel::on_treeview_button_press_event(GdkEventButton* event)
 {
-  if(event->type == GDK_BUTTON_PRESS) //Whatever would cause cellrenderer activation.
-  {
-    //This is really horrible code:
-    //Maybe we can improve the gtkmm API for this:
-
-    //Get the row and column:
-    Gtk::TreeModel::Path path;
-    Gtk::TreeView::Column* pColumn = 0;
-    int cell_x = 0;
-    int cell_y = 0;  
-    bool row_exists = m_TreeView.get_path_at_pos((int)event->x, (int)event->y, path, pColumn, cell_x, cell_y);
-
-    if(row_exists)
-    {
-      //Get the row:
-      Gtk::TreeModel::iterator iterRow = m_refListStore->get_iter(path);
-      if(iterRow)
-      {
-        //Get the column:
-        int tree_col = 0;
-        int col_index = get_count_hidden_system_columns();
-  
-        typedef std::vector<Gtk::TreeView::Column*> type_vecTreeViewColumns;
-        type_vecTreeViewColumns vecColumns = m_TreeView.get_columns();
-        for(type_vecTreeViewColumns::const_iterator iter = vecColumns.begin(); iter != vecColumns.end(); iter++)
-        {
-          if(*iter == pColumn)
-            tree_col = col_index; //Found.
-
-          col_index++;
-        }
-
-        signal_user_activated().emit(iterRow, tree_col);
-      }
-    }
-  }
-
   on_button_press_event_Popup(event);
 }
 
-bool AddDel::on_treeview_columnheader_button_press_event(GdkEventButton* event)
-{
-  //If this is a right-click with the mouse:
-  if( (event->type == GDK_BUTTON_PRESS) && (event->button == 3) )
-  {
-    if(m_pColumnHeaderPopup)
-    {
-      m_pColumnHeaderPopup->popup(event->button, event->time);
-      return true; //It has been handled.
-    }
-    else
-    {
-      //Default popup:
-      //TODO: We might want to disable this sometimes, because it could be useless sometimes.
-      
-    }
-  }
-
-  return false;
-}
-
 bool AddDel::on_treeview_column_drop(Gtk::TreeView* /* treeview */, Gtk::TreeViewColumn* /* column */, Gtk::TreeViewColumn* /* prev_column */, Gtk::TreeViewColumn* /* next_column */)
 {
   return true;
@@ -1366,8 +1381,6 @@ guint AddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRender
   //Save the extra ID, using the title if the column_id is empty:
   pViewColumn->set_column_id( (column_id.empty() ? title : column_id) );
 
-  //TODO pViewColumn->signal_button_press_event().connect( sigc::mem_fun(*this, &AddDel::on_treeview_columnheader_button_press_event) );
-
   return cols_count;
 }
 
@@ -1403,16 +1416,6 @@ AddDel::type_vecStrings AddDel::get_columns_order() const
   return m_vecColumnIDs;
 }
 
-void AddDel::set_column_header_popup(Gtk::Menu& popup)
-{
-  m_pColumnHeaderPopup = &popup;
-}
-
- void AddDel::set_allow_column_chooser(bool value)
-{
-  m_allow_column_chooser = value;
-}
-
 void AddDel::set_auto_add(bool value)
 {
   m_auto_add = value;
