@@ -20,6 +20,7 @@
 
 #include "datawidget.h"
 #include "entryglom.h"
+#include "checkglom.h"
 #include "labelglom.h"
 #include "comboentryglom.h"
 #include "comboglom.h"
@@ -32,6 +33,7 @@
 #include "dialog_choose_date.h"
 #include "../layout_item_dialogs/dialog_field_layout.h"
 #include <glom/libglom/utils.h>
+#include <glom/libglom/glade_utils.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -53,13 +55,14 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
   const Glib::ustring title = field->get_title_or_name();
   if(glom_type == Field::TYPE_BOOLEAN)
   {
-    Gtk::CheckButton* checkbutton = Gtk::manage( new Gtk::CheckButton( title ) );
+    CheckGlom* checkbutton = Gtk::manage( new CheckGlom( title ) );
     checkbutton->show();
     checkbutton->signal_toggled().connect( sigc::mem_fun(*this, &DataWidget::on_widget_edited)  );
 
     //TODO: entry->signal_user_requested_layout().connect( sigc::mem_fun(*this, &DataWidget::on_child_user_requested_layout );
 
     m_child = checkbutton;
+    pFieldWidget = checkbutton;
 
     m_label.set_text( Glib::ustring() ); //It is not used.
   }
@@ -83,7 +86,9 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
   // Use hildon widgets for date and time on maemo
   else
   {
-    m_label.set_label(title);
+    //The GNOME HIG says that labels should have ":" at the end:
+    //http://library.gnome.org/devel/hig-book/stable/design-text-labels.html.en
+    m_label.set_label(title + ":");
     m_label.set_alignment(0);
     m_label.show();
 
@@ -162,7 +167,8 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
         pFieldWidget = entry;
       }
 
-      pFieldWidget->set_layout_item( get_layout_item(), table_name); //TODO_Performance: We only need this for the numerical format.
+      if(pFieldWidget)
+        pFieldWidget->set_layout_item( get_layout_item(), table_name); //TODO_Performance: We only need this for the numerical format.
     }
   }
 
@@ -185,6 +191,9 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
 
   if(m_child)
   {
+    //Use the text formatting:
+    apply_formatting(*m_child, field->get_formatting_used());
+    
     bool child_added = false; //Don't use an extra container unless necessary.
     const bool field_used_in_relationship_to_one = document->get_field_used_in_relationship_to_one(table_name, field->get_name());
 
@@ -206,6 +215,7 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
     {
       //Let the user choose a date from a calendar dialog:
       Gtk::Button* button_date = Gtk::manage(new Gtk::Button(_("..."))); //TODO: A better label/icon for "Choose Date". 
+      button_date->set_tooltip_text(_("Choose a date from an on-screen calendar.")); 
       button_date->show();
       hbox_parent->pack_start(*button_date);
       button_date->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_choose_date));
@@ -215,10 +225,12 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
     {
       //Add buttons for related record navigation:
       Gtk::Button* button_go_to_details = Gtk::manage(new Gtk::Button(Gtk::Stock::OPEN));
+      button_go_to_details->set_tooltip_text(_("Open the record identified by this ID, in the other table."));
       hbox_parent->pack_start(*button_go_to_details);
       button_go_to_details->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_open_details));
 
       Gtk::Button* button_select = Gtk::manage(new Gtk::Button(Gtk::Stock::FIND));
+      button_select->set_tooltip_text(_("Enter search criteria to identify records in the other table, to choose an ID for this field."));
       hbox_parent->pack_start(*button_select);
       button_select->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_select_id));
     }
@@ -344,7 +356,7 @@ int DataWidget::get_suitable_width(const sharedptr<const LayoutItem_Field>& fiel
     }
     case(Field::TYPE_TIME):
     {
-      Gnome::Gda::Time time = {0, 0, 0, 0};
+      Gnome::Gda::Time time = {0, 0, 0, 0, 0};
       time.hour = 24;
       time.minute = 59;
       time.second = 59;
@@ -546,6 +558,11 @@ sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& ta
 }
 
 sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& start_field)
+{
+  return offer_field_list (table_name, start_field, get_document(), get_application());
+}
+
+sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& start_field, Document_Glom* document, App_Glom* app)
 {
   sharedptr<LayoutItem_Field> result;
 
@@ -554,7 +571,7 @@ sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& ta
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
-    refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_choose_field");
+    refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_field");
   }
   catch(const Gnome::Glade::XmlError& ex)
   {
@@ -563,7 +580,7 @@ sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& ta
   }
 #else
   std::auto_ptr<Gnome::Glade::XmlError> error;
-  refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_choose_field", "", error);
+  refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_field", "", error);
   if(error.get())
   {
     std::cerr << error->what() << std::endl;
@@ -576,8 +593,8 @@ sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& ta
 
   if(dialog)
   {
-    dialog->set_document(get_document(), table_name, start_field);
-    dialog->set_transient_for(*get_application());
+    dialog->set_document(document, table_name, start_field);
+    dialog->set_transient_for(*app);
     int response = dialog->run();
     dialog->hide();
     if(response == Gtk::RESPONSE_OK)
@@ -599,7 +616,7 @@ sharedptr<LayoutItem_Field> DataWidget::offer_field_layout(const sharedptr<const
 
   try
   {
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_layout_field_properties");
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_layout_field_properties");
 
     Dialog_FieldLayout* dialog = 0;
     refXml->get_widget_derived("dialog_layout_field_properties", dialog);
@@ -694,6 +711,7 @@ App_Glom* DataWidget::get_application()
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 void DataWidget::on_child_layout_item_added(LayoutWidgetBase::enumType item_type)
 {
+  //Tell the parent widget that this widget wants to add an item:
   signal_layout_item_added().emit(item_type);
 }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
@@ -735,7 +753,7 @@ void DataWidget::on_button_choose_date()
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
-    refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_choose_date");
+    refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_date");
   }
   catch(const Gnome::Glade::XmlError& ex)
   {
@@ -744,7 +762,7 @@ void DataWidget::on_button_choose_date()
   }
 #else
   std::auto_ptr<Gnome::Glade::XmlError> error;
-  refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_choose_date", "", error);
+  refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_date", "", error);
   if(error.get())
   {
     std::cerr << error->what() << std::endl;
@@ -775,7 +793,7 @@ void DataWidget::on_button_choose_date()
   }
 }
 
-void DataWidget::on_self_style_changed(const Glib::RefPtr<Gtk::Style>& style)
+void DataWidget::on_self_style_changed(const Glib::RefPtr<Gtk::Style>& /* style */)
 {
   sharedptr<LayoutItem_Field> layoutField = sharedptr<LayoutItem_Field>::cast_dynamic(get_layout_item());
   set_child_size_by_field(layoutField);
@@ -792,7 +810,7 @@ bool DataWidget::offer_related_record_id_find(Gnome::Gda::Value& chosen_id)
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
-    refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_find_id");
+    refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom.glade"), "dialog_find_id");
   }
   catch(const Gnome::Glade::XmlError& ex)
   {
@@ -801,7 +819,7 @@ bool DataWidget::offer_related_record_id_find(Gnome::Gda::Value& chosen_id)
   }
 #else
   std::auto_ptr<Gnome::Glade::XmlError> error;
-  refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_find_id", "", error);
+  refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom.glade"), "dialog_find_id", "", error);
   if(error.get())
   {
     std::cerr << error->what() << std::endl;
@@ -830,7 +848,7 @@ bool DataWidget::offer_related_record_id_find(Gnome::Gda::Value& chosen_id)
     else
       g_warning("get_layout_item() was not a LayoutItem_Field");
 
-    dialog->init_db_details(related_table_name);
+    dialog->init_db_details(related_table_name, get_document()->get_active_layout_platform());
 
 
     int response = dialog->run();
@@ -848,4 +866,5 @@ bool DataWidget::offer_related_record_id_find(Gnome::Gda::Value& chosen_id)
   return result;
 }
 
+
 } //namespace Glom
