@@ -21,19 +21,24 @@
 #ifndef GLOM_UTILITY_WIDGETS_DATAWIDGET_H
 #define GLOM_UTILITY_WIDGETS_DATAWIDGET_H
 
+#include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
+
 #include "placeholder.h"
-#include "layoutwidgetbase.h"
+#include "layoutwidgetmenu.h"
 #include <gtkmm/label.h>
-#include "../data_structure/field.h"
-#include "../document/document_glom.h"
-#include "../data_structure/layout/layoutitem_field.h"
+#include <glom/libglom/data_structure/field.h>
+#include <glom/libglom/document/document_glom.h>
+#include <glom/libglom/data_structure/layout/layoutitem_field.h>
 #include "../mode_data/treestore_layout.h" //Forthe enum.
 
+namespace Glom
+{
+
 class App_Glom;
 
 class DataWidget
  : public Gtk::EventBox,
-   public LayoutWidgetBase,
+   public LayoutWidgetMenu,
    public View_Composite_Glom
 {
 public:
@@ -55,9 +60,12 @@ public:
   virtual void set_editable(bool editable = true);
   virtual void set_viewable(bool viewable = true);
 
+  static sharedptr<LayoutItem_Field> offer_field_list(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& start_field, Document_Glom* document, App_Glom* app);
   sharedptr<LayoutItem_Field> offer_field_list(const Glib::ustring& table_name);
   sharedptr<LayoutItem_Field> offer_field_list(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& start_field);
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   sharedptr<LayoutItem_Field> offer_field_layout(const sharedptr<const LayoutItem_Field>& start_field);
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 
   /// Get the actual child widget used to show the data:
   Gtk::Widget* get_data_child_widget();
@@ -74,19 +82,31 @@ protected:
 
   //Overrides of default signal handlers:
   void on_widget_edited(); //From Gtk::Entry, or Gtk::CheckButton.
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   virtual bool on_button_press_event(GdkEventButton* event); //override.
   virtual void on_child_user_requested_layout();
   virtual void on_child_user_requested_layout_properties();
   virtual void on_child_layout_item_added(LayoutWidgetBase::enumType item_type);
+#endif // !GLOM_ENABLE_CLIENT_ONLY
   void on_button_open_details();
   void on_button_select_id();
+  void on_button_choose_date();
 
+  // Don't call it on_style_changed, otherwise we would override a virtual
+  // function from Gtk::Widget. We could indeed do that, but we do it with
+  // a normal signal handler, because we have to do it this way anyway in
+  // case default signal handlers have been disabled in glibmm.
+  void on_self_style_changed(const Glib::RefPtr<Gtk::Style>& style);
+
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   virtual void on_menupopup_activate_layout(); //override
   virtual void on_menupopup_activate_layout_properties(); //override
   //virtual void on_menupopup_add_item(LayoutWidgetBase::enumType item);
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 
   App_Glom* get_application();
 
+  void set_child_size_by_field(const sharedptr<const LayoutItem_Field>& field);
   int get_suitable_width(const sharedptr<const LayoutItem_Field>& field_layout);
 
   /** Show a dialog with a Find so that the user can choose an ID value to indicate the related record.
@@ -100,5 +120,7 @@ protected:
   Gtk::Widget* m_child;
 };
 
+} //namespace Glom
+
 #endif //GLOM_UTILITY_WIDGETS_DATAWIDGET_H
 
