@@ -263,20 +263,28 @@ void DbTreeModelRow::fill_values_if_necessary(DbTreeModel& model, int row)
     if((row < (int)model.m_data_model_rows_count) && model.m_gda_datamodel)
     {
       Glib::RefPtr<Gnome::Gda::DataModelIter> iter = model.m_gda_datamodel->create_iter();
-      if(iter) {
-        iter->set_at_row(row);
+      if(iter)
+      {
+        iter->move_to_row(row);
 
         //It is a row from the database;
         const int cols_count = model.m_data_model_columns_count;
         for(int i = 0; i < cols_count; ++i)
         {
-          Glib::RefPtr<Gnome::Gda::Parameter> param = iter->get_param_for_column(i);
-          m_db_values[i] = param->get_value();
+          Glib::RefPtr<const Gnome::Gda::Holder> holder = iter->get_holder_for_field(i);
+          if(holder)
+            m_db_values[i] = holder->get_value(); //TODO_gda: Why not just use get_value_at()?
+          else
+            std::cerr << "DbTreeModelRow::fill_values_if_necessary(): NULL Gnome::Gda::Holder for field=" << i << std::endl;
+
           //std::cout << "  debug: col=" << i << ", GType=" << m_db_values[i].get_value_type() << std::endl;
         }
 
-        Glib::RefPtr<Gnome::Gda::Parameter> param = iter->get_param_for_column(model.m_column_index_key);
-        m_key = param->get_value();
+        Glib::RefPtr<const Gnome::Gda::Holder> holder = iter->get_holder_for_field(model.m_column_index_key);
+        if(holder)
+          m_key = holder->get_value();  //TODO_gda: Why not just use get_value_at()?
+        else
+          std::cerr << "DbTreeModelRow::fill_values_if_necessary(): NULL Gnome::Gda::Holder for key field" << std::endl;
 
         m_extra = false;
         m_removed = false;
@@ -465,16 +473,13 @@ bool DbTreeModel::refresh_from_database(const FoundSet& found_set)
 #endif //GLIBMM_EXCEPTIONS_ENABLED
     }
 
+    Glib::RefPtr<Gnome::Gda::SqlParser> parser = m_connection->get_gda_connection()->create_parser();
+    Glib::RefPtr<Gnome::Gda::Statement> stmt = parser->parse_string(sql_query);
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
     {
-      //Specify the ITER_MODEL_ONLY parameter, so that libgda only gets the rows that we actually use.
-      Glib::RefPtr<Gnome::Gda::ParameterList> params = Gnome::Gda::ParameterList::create();
-      Gnome::Gda::Value value;
-      value.set(true);
-      params->add_parameter("ITER_MODEL_ONLY", value);
-
-      m_gda_datamodel = m_connection->get_gda_connection()->execute_select_command(sql_query, params);
+      //Specify the STATEMENT_MODEL_CURSOR, so that libgda only gets the rows that we actually use.   
+      m_gda_datamodel = m_connection->get_gda_connection()->statement_execute_select(stmt, Gnome::Gda::STATEMENT_MODEL_CURSOR);
 
       if(app && app->get_show_sql_debug())
         std::cout << "  Debug: DbTreeModel::refresh_from_database(): The query execution has finished." << std::endl;
@@ -500,7 +505,7 @@ bool DbTreeModel::refresh_from_database(const FoundSet& found_set)
     }
 #else
     std::auto_ptr<Glib::Error> error;
-    m_gda_datamodel = m_connection->get_gda_connection()->execute_select_command(sql_query, error);
+    m_gda_datamodel = m_connection->get_gda_connection()->statement_execute_select(stmt, set, error);
     if(error.get())
     {
       m_gda_datamodel.clear(); //So that it is 0, so we can handle it below.
@@ -518,7 +523,7 @@ bool DbTreeModel::refresh_from_database(const FoundSet& found_set)
     }
     else
     {
-      //This doesn't work with ITER_MODEL_ONLY: const int count = m_gda_datamodel->get_n_rows();
+      //This doesn't work with cursor-based models: const int count = m_gda_datamodel->get_n_rows();
       //because rows count is -1 until we have iterated to the last row.
       const Glib::ustring sql_query_without_sort = Utils::build_sql_select_with_where_clause(m_found_set.m_table_name, m_column_fields, m_found_set.m_where_clause, m_found_set.m_extra_join, type_sort_clause(), m_found_set.m_extra_group_by);
       const int count = Base_DB::count_rows_returned_by(sql_query_without_sort);
@@ -1125,11 +1130,13 @@ void DbTreeModel::get_record_counts(gulong& total, gulong& found) const
       //Ask the database how many records there are in the whole table:
       //TODO: Apparently, this is very slow:
       const Glib::ustring sql_query = "SELECT count(*) FROM \"" + m_found_set.m_table_name + "\"";
+      Glib::RefPtr<Gnome::Gda::SqlParser> parser = m_connection->get_gda_connection()->create_parser();
+      Glib::RefPtr<Gnome::Gda::Statement> stmt = parser->parse_string(sql_query);
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
-      Glib::RefPtr<Gnome::Gda::DataModel> datamodel = m_connection->get_gda_connection()->execute_select_command(sql_query);
+      Glib::RefPtr<Gnome::Gda::DataModel> datamodel = m_connection->get_gda_connection()->statement_execute_select(stmt);
 #else
       std::auto_ptr<Glib::Error> error;
-      Glib::RefPtr<Gnome::Gda::DataModel> datamodel = m_connection->get_gda_connection()->execute_select_command(sql_query, error);
+      Glib::RefPtr<Gnome::Gda::DataModel> datamodel = m_connection->get_gda_connection()->statement_execute_select(stmt, error);
       // Ignore error, datamodel presence is checked below
 #endif //GLIBMM_EXCEPTIONS_ENABLED
 
@@ -1138,7 +1145,7 @@ void DbTreeModel::get_record_counts(gulong& total, gulong& found) const
         if(datamodel->get_n_rows())
         {
           Gnome::Gda::Value value = datamodel->get_value_at(0, 0);
-	  total = (gulong)value.get_int64(); //I discovered that it's a int64 by trying it.
+          total = (gulong)value.get_int64(); //I discovered that it's a int64 by trying it.
         }
       }
     }
